# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for an SQLite 3 wrapper Class written for NZBGet
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import sys
from os.path import dirname
from os.path import join
sys.path.insert(0, join(dirname(dirname(__file__)), 'nzbget'))

from Utils import os_path_split
from Utils import tidy_path
from TestBase import TestBase


class TestUtils(TestBase):
    def setUp(self):
        """This method is run once before _each_ test method is executed"""
        super(TestUtils, self).setUp()

    def tearDown(self):
        """This method is run once after _each_ test method is executed"""
        super(TestUtils, self).tearDown()

    def test_os_path_split(self):
        assert os_path_split('C:\\My Directory\\SubDirectory') == [
            'C:',
            'My Directory',
            'SubDirectory',
        ]
        assert os_path_split('relative/nix/pathname') == [
            'relative',
            'nix',
            'pathname',
        ]
        assert os_path_split('/absolute/nix/pathname') == [
            '',
            'absolute',
            'nix',
            'pathname',
        ]
        assert os_path_split('') == []
        assert os_path_split('/') == ['', ]
        assert os_path_split('\\') == ['', ]

        # Weird Paths are fixed
        assert os_path_split('///////////') == ['', ]
        assert os_path_split('\\\\\\\\\\') == ['', ]

        # Trailing slashes are removed
        assert os_path_split('relative/nix/pathname/') == [
            'relative',
            'nix',
            'pathname',
        ]

    def test_tidy_path(self):

        # No Change
        assert tidy_path('C:\\My Directory\\SubDirectory') == \
            'C:\\My Directory\\SubDirectory'

        assert tidy_path('C:\\\\\My Directory\\\\\\SubDirectory  ') == \
            'C:\\My Directory\\SubDirectory'

        assert tidy_path('C:\\') == 'C:\\'
        assert tidy_path('C:\\\\\\') == 'C:\\'
        assert tidy_path('/') == '/'
        assert tidy_path('///////////') == '/'
        assert tidy_path('////path///////') == '/path'
        assert tidy_path('////path/with spaces//////') == \
            '/path/with spaces'

        # Network Paths
        assert tidy_path('\\\\network\\\\path\\ ') == \
            '\\\\network\\path'



# -*- encoding: utf-8 -*-
#
# A scripting wrapper for NZBGet's Scan Scripting
#
# Copyright (C) 2014 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
"""
This class was intended to make writing NZBGet Scripts easier to manage and
write by handling the common error handling and provide the most reused code
in a re-usable container. It was initially written to work with NZBGet v13
but provides most backwards compatibility.

It was designed to be inheritied as a base class requiring you to only write
the main() function which should preform the task you are intending.

It looks after fetching all of the environment variables and will parse
the meta information out of the NZB-File.

It allows you to set variables that other scripts can access if they need to
using the set() and get() variables. This is done through a simply self
maintained hash table type structure within a sqlite database. All the
wrapper functions are already written.  If you call 'set('MYKEY', 1')
you can call get('MYKEY') in another script and continue working

push() functions written to pass information back to nzbget using it's
processing engine.

all exceptions are now automatically handled and logging can be easily
changed from stdout, to stderr or to a file.

Test suite built in (using python-nose) to ensure old global variables
will still work as well as make them easier to access and manipulate.

Some inline documentation was based on content provided at:
   - http://nzbget.net/Extension_scripts


############################################################################
Scan Script Usage/Example
############################################################################

############################################################################
### NZBGET SCAN SCRIPT                                                   ###
#
# Describe your Scan Script here
# Author: Chris Caron <lead2gold@gmail.com>
#

############################################################################
### OPTIONS                                                              ###

#
# Enable NZBGet debug logging (yes, no)
# Debug=no
#

### NZBGET SCAN SCRIPT                                                   ###
############################################################################

from nzbget import ScanScript

# Now define your class while inheriting the rest
class MyScanScript(ScanScript):
    def main(self, *args, **kwargs):

        # Version Checking, Environment Variables Present, etc
        if not self.validate():
            # No need to document a failure, validate will do that
            # on the reason it failed anyway
            return False

        # write all of your code here you would have otherwise put in the
        # script

        # All system environment variables (NZBOP_.*) as well as Post
        # Process script specific content (NZBNP_.*)
        # following dictionary (without the NZBOP_ or NZBNP_ prefix):
        print('TEMPDIR (directory is: %s' % self.get('TEMPDIR'))
        print('DIRECTORY %s' self.get('DIRECTORY'))
        print('FILENAME %s' self.get('FILENAME'))
        print('NZBNAME %s' self.get('NZBNAME'))
        print('CATEGORY %s' self.get('CATEGORY'))
        print('PRIORITY %s' self.get('PRIORITY'))
        print('TOP %s' self.get('TOP'))
        print('PAUSED %s' self.get('PAUSED'))

        # Set any variable you want by any key.  Note that if you use
        # keys that were defined by the system (such as CATEGORY, DIRECTORY,
        # etc, you may have some undesirable results.  Try to avoid reusing
        # system variables already defined (identified above):
        self.set('MY_KEY', 'MY_VALUE')

        # You can fetch it back; this will also set an entry in  the
        # sqlite database for each hash references that can be pulled from
        # another script that simply calls self.get('MY_KEY')
        print(self.get('MY_KEY')) # prints MY_VALUE

        # You can also use push() which is similar to set()
        # except that it interacts with the NZBGet Server and does not use
        # the sqlite database. This can only be reached across other
        # scripts if the calling application is NZBGet itself
        self.push('ANOTHER_KEY', 'ANOTHER_VALUE')

        # You can still however locally retrieve what you set using push()
        # with the get() function
        print(self.get('ANOTHER_KEY')) # prints ANOTHER_VALUE

        # Your script configuration files (NZBNP_.*) are here in this
        # dictionary (again without the NZBNP_ prefix):
        # assume you defined `Debug=no` in the first 10K of your ScanScript
        # NZBGet translates this to `NZBNP_DEBUG` which can be retrieved
        # as follows:
        print('DEBUG %s' self.get('DEBUG'))

        # Returns have been made easy.  Just return:
        #   * True if everything was successful
        #   * False if there was a problem
        #   * None if you want to report that you've just gracefully
                  skipped processing (this is better then False)
                  in some circumstances. This is neither a failure or a
                  success status.

        # Feel free to use the actual exit codes as well defined by
        # NZBGet on their website.  They have also been defined here
        # from nzbget.ScriptBase import EXIT_CODE

        return True

# Call your script as follows:
if __name__ == "__main__":
    from sys import exit

    # Create an instance of your Script
    myscript = MyScanScript()

    # call run() and exit() using it's returned value
    exit(myscript.run())

"""
import re
from os import chdir
from os import environ
from os.path import isdir
from os.path import basename
from os.path import abspath

# Relative Includes
from .ScriptBase import ScriptBase
from .ScriptBase import SCRIPT_MODE
from .ScriptBase import NZBGET_BOOL_FALSE
from .ScriptBase import PRIORITY
from .ScriptBase import PRIORITIES

# Environment variable that prefixes all NZBGET options being passed into
# scripts with respect to the NZB-File (used in Scan Scripts)
SCAN_ENVIRO_ID = 'NZBNP_'

# Precompile Regulare Expression for Speed
SCAN_OPTS_RE = re.compile('^%s([A-Z0-9_]+)$' % SCAN_ENVIRO_ID)

class ScanScript(ScriptBase):
    def __init__(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Multi-Script Support
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if not hasattr(self, 'script_dict'):
            # Only define once
            self.script_dict = {}
        self.script_dict[SCRIPT_MODE.SCAN] = self

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Initialize Parent
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        super(ScanScript, self).__init__(*args, **kwargs)

    def scan_init(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Fetch Script Specific Arguments
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        directory = kwargs.get('directory')
        nzbname = kwargs.get('nzbname')
        filename = kwargs.get('filename')
        category = kwargs.get('category')
        priority = kwargs.get('priority')
        top = kwargs.get('top')
        paused = kwargs.get('paused')
        parse_nzbfile = kwargs.get('parse_nzbfile')
        use_database = kwargs.get('use_database')

        # Fetch/Load Scan Script Configuration
        script_config = dict([(SCAN_OPTS_RE.match(k).group(1), v.strip()) \
               for (k, v) in environ.items() if SCAN_OPTS_RE.match(k)])

        if self.vvdebug:
            # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
            # Print Global Script Varables to help debugging process
            # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
            for k, v in script_config.items():
                self.logger.vvdebug('%s%s=%s' % (SCAN_ENVIRO_ID, k, v))

        # Merge Script Configuration With System Config
        self.system = dict(script_config.items() + self.system.items())

        # self.directory
        # This is the path to the destination directory for downloaded files.
        if directory is None:
            self.directory = environ.get(
                '%sDIRECTORY' % SCAN_ENVIRO_ID,
            )
        else:
            self.directory = directory

        # self.nzbname
        # User-friendly name of processed nzb-file as it is displayed by the
        # program.  The file path and extension are removed.  If download was
        # renamed, this parameter reflects the new name.
        if nzbname is None:
            self.nzbname = environ.get(
                '%sNZBNAME' % SCAN_ENVIRO_ID,
            )
        else:
            self.nzbname = nzbname

        # self.filename
        # Name of file to be processed
        if filename is None:
            self.filename = environ.get(
                '%sFILENAME' % SCAN_ENVIRO_ID,
            )
        else:
            self.filename = filename

        # self.category
        # Category assigned to nzb-file (can be empty string).
        if category is None:
            self.category = environ.get(
                '%sCATEGORY' % SCAN_ENVIRO_ID,
            )
        else:
            self.category = category

        # self.priority
        # The priority of the nzb file being scanned
        if priority is None:
            self.priority = environ.get(
                '%sPRIORITY' % SCAN_ENVIRO_ID,
            )
        else:
            self.priority = priority

        # self.top
        # Flag indicating that the file will be added to the top of queue
        if top is None:
            self.top = environ.get(
                '%sTOP' % SCAN_ENVIRO_ID,
            )
        else:
            self.top = top

        # self.paused
        # Flag indicating that the file will be added as paused
        if paused is None:
            self.paused = environ.get(
                '%sPAUSED' % SCAN_ENVIRO_ID,
            )
        else:
            self.paused = paused

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Error Handling
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if self.filename:
            # absolute path names
            self.filename = abspath(self.filename)

            if parse_nzbfile:
                # Initialize information fetched from NZB-File
                # We intentionally allow existing nzbheaders to over-ride
                # any found in the nzbfile
                self.nzbheaders = dict(
                    self.parse_nzbfile(
                        self.filename, check_queued=True)\
                        .items() + self.pull_dnzb().items(),
                )

        if self.directory:
            # absolute path names
            self.directory = abspath(self.directory)

        if not (self.directory and isdir(self.directory)):
            self.logger.debug('Process directory is missing: %s' % \
                self.directory)
        else:
            try:
                chdir(self.directory)
            except OSError:
                self.logger.debug('Process directory is not accessible: %s' % \
                    self.directory)

        # Priority
        if not isinstance(self.priority, int):
            try:
                self.priority = int(self.priority)
            except:
                self.priority = PRIORITY.NORMAL

        if self.priority not in PRIORITIES:
            self.priority = PRIORITY.NORMAL

        # Top
        try:
            self.top = bool(int(self.top))
        except:
            self.top = False

        # Paused
        try:
            self.paused = bool(int(self.paused))
        except:
            self.paused = False

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Enforce system/global variables for script processing
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        self.system['DIRECTORY'] = self.directory
        if self.directory is not None:
            environ['%sDIRECTORY' % SCAN_ENVIRO_ID] = self.directory

        self.system['NZBNAME'] = self.nzbname
        if self.nzbname is not None:
            environ['%sNZBNAME' % SCAN_ENVIRO_ID] = self.nzbname

        self.system['FILENAME'] = self.filename
        if self.filename is not None:
            environ['%sFILENAME' % SCAN_ENVIRO_ID] = self.filename

        self.system['CATEGORY'] = self.category
        if self.category is not None:
            environ['%sCATEGORY' % SCAN_ENVIRO_ID] = self.category

        self.system['PRIORITY'] = self.priority
        if self.priority is not None:
            environ['%sPRIORITY' % SCAN_ENVIRO_ID] = str(self.priority)

        self.system['TOP'] = self.top
        if self.top is not None:
            environ['%sTOP' % SCAN_ENVIRO_ID] = str(int(self.top))

        self.system['PAUSED'] = self.paused
        if self.paused is not None:
            environ['%sPAUSED' % SCAN_ENVIRO_ID] = str(int(self.paused))

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Create Database for set() and get() operations
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if use_database:
            # database_key is inherited in the parent class
            # future calls of set() and get() will allow access
            # to the database now
            try:
                self.database_key = basename(self.filename)
                self.logger.info('Connected to SQLite Database')
            except AttributeError:
                pass

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Debug Flag Check
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def scan_debug(self, *args, **kargs):
        """Uses the environment variables to detect if debug mode is set
        """
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Debug Handling
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        return self.parse_bool(
            environ.get('%sDEBUG' % SCAN_ENVIRO_ID, NZBGET_BOOL_FALSE),
        )

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Sanity
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def scan_sanity_check(self, *args, **kargs):
        """Sanity checking to ensure this really is a post_process script
        """
        from PostProcessScript import POSTPROC_ENVIRO_ID
        return ('%sDIRECTORY' % POSTPROC_ENVIRO_ID not in environ) and \
               ('%sDIRECTORY' % SCAN_ENVIRO_ID in environ)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Validatation
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def scan_validate(self, keys=None, min_version=11, *args, **kargs):
        """validate against environment variables
        """
        is_okay = super(ScanScript, self)._validate(
            keys=keys,
            min_version=min_version,
        )
        return is_okay

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # File Retrieval
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def scan_get_files(self, search_dir=None, regex_filter=None,
                       prefix_filter=None, suffix_filter=None,
                       fullstats=False, *args, **kargs):
        """a wrapper to the get_files() function defined in the inherited class
           the only difference is the search_dir automatically uses the
           defined download `directory` as a default (if not specified).
        """
        if search_dir is None:
            search_dir = self.directory

        return super(ScanScript, self)._get_files(
            search_dir=search_dir, *args, **kargs)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Set/Control Functions (also passes data back to NZBGet)
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def push_nzbname(self, nzbname=None):
        if nzbname:
            # Update local nzbname
            self.nzbname = nzbname

            # Accomodate other environmental variables
            self.system['NZBNAME'] = self.nzbname
            environ['%sNZBNAME' % SCAN_ENVIRO_ID] = self.nzbname

        # Alert NZBGet of Change
        return self._push(
            key='NZBNAME',
            value=self.nzbname,
        )

    def push_category(self, category=None):
        if category:
            # Update local category
            self.category = category

            # Accomodate other environmental variables
            self.system['CATEGORY'] = self.category
            environ['%sCATEGORY' % SCAN_ENVIRO_ID] = self.category

        # Alert NZBGet of Change
        return self._push(
            key='CATEGORY',
            value=self.category,
        )

    def push_priority(self, priority=None):

        if priority is not None:
            if priority not in PRIORITIES:
                return False

            # Update local priority
            self.priority = priority

            # Accomodate other environmental variables
            self.system['PRIORITY'] = self.priority
            environ['%sPRIORITY' % SCAN_ENVIRO_ID] = str(self.priority)

        # Alert NZBGet of Change
        return self._push(
            key='PRIORITY',
            value=self.priority,
        )

    def push_top(self, top=None):

        if top is not None:
            # Update local priority
            try:
                self.top = bool(int(top))
            except:
                return False

            # Accomodate other environmental variables
            self.system['TOP'] = self.top
            environ['%sTOP' % SCAN_ENVIRO_ID] = str(int(self.top))

        # Alert NZBGet of Change
        return self._push(
            key='TOP',
            # Convert bool to int for response
            value=int(self.top),
        )

    def push_paused(self, paused=None):

        if paused is not None:
            # Update local priority
            try:
                self.paused = bool(int(paused))
            except:
                return False

            # Accomodate other environmental variables
            self.system['PAUSED'] = self.paused
            environ['%sPAUSED' % SCAN_ENVIRO_ID] = str(int(self.paused))

        # Alert NZBGet of Change
        return self._push(
            key='PAUSED',
            # Convert bool to int for response
            value=int(self.paused),
        )



# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for the SchedulerScript Class
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import os

from TestBase import TestBase
from TestBase import TEMP_DIRECTORY

from nzbget.ScriptBase import CFG_ENVIRO_ID
from nzbget.ScriptBase import SYS_ENVIRO_ID
from nzbget.SchedulerScript import SchedulerScript
from nzbget.SchedulerScript import SCHEDULER_ENVIRO_ID
from nzbget.SchedulerScript import TASK_ENVIRO_ID

from nzbget.Logger import VERY_VERBOSE_DEBUG

# Some constants to work with
TASKID = "1"
TASK_PARAM = 'MyScript.py'
TASK_TIME = '00:00,00:15'


class TestSchedulerScript(TestBase):
    def setUp(self):
        """This method is run once before _each_ test method is executed"""
        super(TestSchedulerScript, self).setUp()

        # Create some environment variables
        os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID] = TASKID
        os.environ['%s%s%s_PARAM' % (
            SCHEDULER_ENVIRO_ID, TASK_ENVIRO_ID, TASKID)] = TASK_PARAM
        os.environ['%s%s%s_TIME' % (
            SCHEDULER_ENVIRO_ID, TASK_ENVIRO_ID, TASKID)] = TASK_TIME

    def tearDown(self):
        """This method is run once after _each_ test method is executed"""
        # Eliminate any variables defined
        del os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID]
        del os.environ['%s%s%s_PARAM' % (
            SCHEDULER_ENVIRO_ID, TASK_ENVIRO_ID, TASKID)]
        del os.environ['%s%s%s_TIME' % (
            SCHEDULER_ENVIRO_ID, TASK_ENVIRO_ID, TASKID)]

        # common
        super(TestSchedulerScript, self).tearDown()

    def test_environment_varable_init(self):
        """
        Testing NZBGet Script initialization using environment variables
        """
        # a NZB Logger set to False uses stderr
        script = SchedulerScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.taskid == int(TASKID)

        assert script.system['TASKID'] == int(TASKID)
        assert script.system['%s%s_PARAM' % (
            TASK_ENVIRO_ID, TASKID)] == TASK_PARAM
        assert script.system['%s%s_TIME' % (
            TASK_ENVIRO_ID, TASKID)] == TASK_TIME

        assert script.get('TEMPDIR') == TEMP_DIRECTORY
        assert script.get('TASKID') == int(TASKID)
        assert script.get('%s%s_PARAM' % (
            TASK_ENVIRO_ID, TASKID)) == TASK_PARAM
        assert script.get('%s%s_TIME' % (
            TASK_ENVIRO_ID, TASKID)) == TASK_TIME

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG

        assert os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID] == TASKID
        assert os.environ['%s%s%s_PARAM' % (
            SCHEDULER_ENVIRO_ID, TASK_ENVIRO_ID, TASKID)] == TASK_PARAM
        assert os.environ['%s%s%s_TIME' % (
            SCHEDULER_ENVIRO_ID, TASK_ENVIRO_ID, TASKID)] == TASK_TIME

    def test_environment_override(self):
        """
        Testing NZBGet Script initialization using forced variables that
        should take priority over global ones
        """
        taskid = int(TASKID) + 1
        script = SchedulerScript(
            logger=False,
            debug=VERY_VERBOSE_DEBUG,

            taskid=taskid,
        )

        assert script.taskid == taskid

        assert script.system['TEMPDIR'] == TEMP_DIRECTORY
        assert script.system['TASKID'] == taskid

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG

        assert os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sTASKID' % SCHEDULER_ENVIRO_ID] == str(taskid)

    def test_get_task(self):

        script = SchedulerScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        task = script.get_task()
        print str(task)
        assert len(task) == 2

    def test_set_and_get(self):
        # a NZB Logger set to False uses stderr
        script = SchedulerScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        KEY = 'MY_VAR'
        VALUE = 'MY_VALUE'

        # Value doe snot exist yet
        assert script.get(KEY) is None
        assert script.get(KEY, 'Default') == 'Default'
        assert script.set(KEY, VALUE) is True
        assert script.get(KEY, 'Default') == VALUE

    def test_config_varable_init(self):
        valid_entries = {
            'MY_CONFIG_ENTRY': 'Option A',
            'ENTRY_WITH_234_NUMBERS': 'Option B',
            '123443': 'Option C',
        }
        invalid_entries = {
            'CONFIG_ENtry_skipped': 'Option',
            'CONFIG_ENtry_$#': 'Option',
            # Empty
            '': 'Option',
        }
        for k, v in valid_entries.items():
            os.environ['%s%s' % (CFG_ENVIRO_ID, k)] = v
        for k, v in invalid_entries.items():
            os.environ['%s%s' % (CFG_ENVIRO_ID, k)] = v

        script = SchedulerScript()
        for k, v in valid_entries.items():
            assert k in script.config
            assert script.config[k] == v

        for k in invalid_entries.keys():
            assert k not in script.config

        # Cleanup
        for k, v in valid_entries.items():
            del os.environ['%s%s' % (CFG_ENVIRO_ID, k)]
        for k, v in invalid_entries.items():
            del os.environ['%s%s' % (CFG_ENVIRO_ID, k)]



# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for an SQLite 3 wrapper Class written for NZBGet
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
from TestBase import TestBase
from TestBase import TEMP_DIRECTORY

from os import unlink
from os.path import join

from nzbget.Database import Database
from nzbget.Database import Category
from nzbget.Database import NZBGET_DATABASE_VERSION
from nzbget.Logger import VERY_VERBOSE_DEBUG

# Temporary Directory
DATABASE = join(TEMP_DIRECTORY, 'nzbget_test.db')

KEY = 'The.Perfect.Name.nzb'


class TestDatabase(TestBase):
    def tearDown(self):
        """This method is run once after _each_ test method is executed"""
        try:
            unlink(DATABASE)
        except:
            pass

        # common
        super(TestDatabase, self).tearDown()

    def test_schema_init(self):

        db = Database(
            container=KEY,
            database=DATABASE,
            reset=True,

            debug=VERY_VERBOSE_DEBUG,
        )
        assert db._get_version() == NZBGET_DATABASE_VERSION
        assert db._schema_okay()

    def test_key_manip(self):

        db = Database(
            container=KEY,
            database=DATABASE,
            reset=True,

            debug=VERY_VERBOSE_DEBUG,
        )
        # New Keys
        assert db.set('MY_KEY', 'MY_VALUE')
        assert db.get('MY_KEY') == 'MY_VALUE'
        # Updates
        assert db.set('MY_KEY', 'MY_NEW_VALUE')
        assert db.get('MY_KEY') == 'MY_NEW_VALUE'
        # Other Keys
        assert db.set('MY_OTHER_KEY', 'MY_OTHER_VALUE')
        assert db.get('MY_OTHER_KEY') == 'MY_OTHER_VALUE'

        del db
        # Content saved across sessions
        db = Database(
            container=KEY,
            database=DATABASE,

            debug=VERY_VERBOSE_DEBUG,
        )
        assert db.get('MY_KEY') == 'MY_NEW_VALUE'
        assert db.get('MY_OTHER_KEY') == 'MY_OTHER_VALUE'

        # Removing
        assert db.unset('MY_OTHER_KEY')
        assert db.get('MY_OTHER_KEY', 'MISSING') == 'MISSING'

    def test_category_manip(self):

        db = Database(
            container=KEY,
            database=DATABASE,
            reset=True,

            debug=VERY_VERBOSE_DEBUG,
        )
        # New Keys
        assert db.set('MY_KEY', 'MY_VALUE')
        assert db.get('MY_KEY') == 'MY_VALUE'
        assert db.set('MY_KEY', 'MY_NEW_VALUE', category=Category.NZB)
        # No change in key
        assert db.get('MY_KEY') == 'MY_VALUE'
        # However, different category has different key mapped
        assert db.get('MY_KEY', category=Category.NZB) == 'MY_NEW_VALUE'

        # Updates
        assert db.set('MY_KEY', 'ANOTHER_VALUE', category=Category.NZB)
        assert db.get('MY_KEY', category=Category.NZB) == 'ANOTHER_VALUE'
        del db

        # Content saved across sessions
        db = Database(
            container=KEY,
            database=DATABASE,

            debug=VERY_VERBOSE_DEBUG,
        )
        assert db.get('MY_KEY') == 'MY_VALUE'
        assert db.get('MY_KEY', category=Category.NZB) == 'ANOTHER_VALUE'

    def test_key_purges01(self):

        db = Database(
            container=KEY,
            database=DATABASE,
            reset=True,

            debug=VERY_VERBOSE_DEBUG,
        )

        assert db.set('MY_KEY', 'MY_VALUE')
        assert db.get('MY_KEY') == 'MY_VALUE'

        assert db.set('MY_OTHER_KEY', 'MY_OTHER_VALUE')
        assert db.get('MY_OTHER_KEY') == 'MY_OTHER_VALUE'

        # purge entries (0 = all)

    def test_key_purges02(self):

        db = Database(
            container=KEY,
            database=DATABASE,
            reset=True,

            debug=VERY_VERBOSE_DEBUG,
        )

        assert db.set('MY_KEY', 'MY_VALUE')
        assert db.get('MY_KEY') == 'MY_VALUE'

        assert db.set('MY_OTHER_KEY', 'MY_OTHER_VALUE')
        assert db.get('MY_OTHER_KEY') == 'MY_OTHER_VALUE'

        # purge entries (0 = all)
        db.prune(0)
        assert db.get('MY_KEY') is None
        assert db.get('MY_OTHER_KEY') is None



# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for the FeedScript Class
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import os

from TestBase import TestBase
from TestBase import TEMP_DIRECTORY

from nzbget.ScriptBase import CFG_ENVIRO_ID
from nzbget.ScriptBase import SYS_ENVIRO_ID

from nzbget.FeedScript import FeedScript
from nzbget.FeedScript import FEED_ENVIRO_ID

from nzbget.Logger import VERY_VERBOSE_DEBUG

# Some constants to work with
FEEDID = "1"
FEED_FILENAME = 'MyTest.nzb'


class TestFeedScript(TestBase):
    def setUp(self):
        """This method is run once before _each_ test method is executed"""
        super(TestFeedScript, self).setUp()

        # Create some environment variables
        os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sFEEDID' % FEED_ENVIRO_ID] = FEEDID
        os.environ['%sFILENAME' % FEED_ENVIRO_ID] = FEED_FILENAME

    def tearDown(self):
        """This method is run once after _each_ test method is executed"""
        # Eliminate any variables defined
        del os.environ['%sFEEDID' % FEED_ENVIRO_ID]
        del os.environ['%sFILENAME' % FEED_ENVIRO_ID]

        # common
        super(TestFeedScript, self).tearDown()

    def test_environment_varable_init(self):
        """
        Testing NZBGet Script initialization using environment variables
        """
        # a NZB Logger set to False uses stderr
        script = FeedScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.feedid == int(FEEDID)
        assert script.filename == FEED_FILENAME

        assert script.system['FEEDID'] == int(FEEDID)
        assert script.system['FILENAME'] == FEED_FILENAME

        assert script.get('TEMPDIR') == TEMP_DIRECTORY
        assert script.get('FEEDID') == int(FEEDID)
        assert script.get('FILENAME') == FEED_FILENAME

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG

        assert os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sFEEDID' % FEED_ENVIRO_ID] == FEEDID
        assert os.environ['%sFILENAME' % FEED_ENVIRO_ID] == FEED_FILENAME

    def test_environment_override(self):
        """
        Testing NZBGet Script initialization using forced variables that
        should take priority over global ones
        """
        feedid = int(FEEDID) + 1
        filename = 'TEST_OVERLOAD_%s' % FEED_FILENAME
        script = FeedScript(
            logger=False,
            debug=VERY_VERBOSE_DEBUG,
            feedid=feedid,
            filename=filename,
        )

        assert script.feedid == feedid

        assert script.system['TEMPDIR'] == TEMP_DIRECTORY
        assert script.system['FEEDID'] == feedid
        assert script.system['FILENAME'] == filename

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG

        assert os.environ['%sTEMPDIR' % SYS_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sFEEDID' % FEED_ENVIRO_ID] == str(feedid)
        assert os.environ['%sFILENAME' % FEED_ENVIRO_ID] == filename

    def test_get_feed(self):

        script = FeedScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        feed = script.get_feed()
        assert len(feed) == 0

    def test_set_and_get(self):
        # a NZB Logger set to False uses stderr
        script = FeedScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        KEY = 'MY_VAR'
        VALUE = 'MY_VALUE'

        # Value does not exist yet
        assert script.get(KEY) is None
        assert script.get(KEY, 'Default') == 'Default'
        assert script.set(KEY, VALUE) is True
        assert script.get(KEY, 'Default') == VALUE

    def test_config_varable_init(self):
        valid_entries = {
            'MY_CONFIG_ENTRY': 'Option A',
            'ENTRY_WITH_234_NUMBERS': 'Option B',
            '123443': 'Option C',
        }
        invalid_entries = {
            'CONFIG_ENtry_skipped': 'Option',
            'CONFIG_ENtry_$#': 'Option',
            # Empty
            '': 'Option',
        }
        for k, v in valid_entries.items():
            os.environ['%s%s' % (CFG_ENVIRO_ID, k)] = v
        for k, v in invalid_entries.items():
            os.environ['%s%s' % (CFG_ENVIRO_ID, k)] = v

        script = FeedScript()
        for k, v in valid_entries.items():
            assert k in script.config
            assert script.config[k] == v

        for k in invalid_entries.keys():
            assert k not in script.config

        # Cleanup
        for k, v in valid_entries.items():
            del os.environ['%s%s' % (CFG_ENVIRO_ID, k)]
        for k, v in invalid_entries.items():
            del os.environ['%s%s' % (CFG_ENVIRO_ID, k)]



# -*- encoding: utf-8 -*-
#
# A Test Suite (for nose) for the SABnzbd SABPostProcessScript Class
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import os
from os import makedirs
from os.path import basename
from os.path import dirname
from os.path import isfile
from os.path import join

from TestBase import TestBase
from TestBase import TEMP_DIRECTORY

from nzbget.ScriptBase import SAB_ENVIRO_ID
from nzbget.ScriptBase import SHELL_EXIT_CODE

from nzbget.SABPostProcessScript import SABPostProcessScript
from nzbget.SABPostProcessScript import PP_STATUS
from nzbget.Logger import VERY_VERBOSE_DEBUG


from shutil import rmtree

# Some constants to work with
DIRECTORY = TEMP_DIRECTORY
COMPLETE_DIRECTORY = TEMP_DIRECTORY
NZBNAME = 'A.Great.Movie'
NZBFILENAME = join(TEMP_DIRECTORY, 'A.Great.Movie.nzb')
CATEGORY = 'movie'
VERSION = '2.1'
SEARCH_DIR = join(TEMP_DIRECTORY, 'file_listing')

# For validation
STATUS = str(PP_STATUS.SUCCESS)


class TestSABPostProcessScript(TestBase):
    def setUp(self):
        """This method is run once before _each_ test method is executed"""
        super(TestSABPostProcessScript, self).setUp()

        # Create NZBFILE
        _f = open(NZBFILENAME, 'w')
        _f.write("""<?xml version="1.0" encoding="UTF-8"?>
            <!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.1//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.1.dtd">
            <nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">

            <head>
             <meta type="category">Movies &gt; SD</meta>
             <meta type="name">A.Great.Movie.1983.DVDRip.x264-AWESOME</meta>
             <meta type="propername">A Great Movie</meta>
             <meta type="movieyear">1983</meta>
            <!-- extra record used to determine what file we're reading -->
             <meta type="letter">A</meta>
            </head>
          </nzb>""")
        _f.close()

        # Create some environment variables
        os.environ['%sTEMPDIR' % SAB_ENVIRO_ID] = TEMP_DIRECTORY
        os.environ['%sVERSION' % SAB_ENVIRO_ID] = str(VERSION)
        os.environ['%sDIRECTORY' % SAB_ENVIRO_ID] = DIRECTORY
        os.environ['%sCOMPLETE_DIR' % SAB_ENVIRO_ID] = COMPLETE_DIRECTORY
        os.environ['%sNZBNAME' % SAB_ENVIRO_ID] = NZBFILENAME
        os.environ['%sFILENAME' % SAB_ENVIRO_ID] = NZBNAME
        os.environ['%sCAT' % SAB_ENVIRO_ID] = CATEGORY
        os.environ['%sPP_STATUS' % SAB_ENVIRO_ID] = STATUS

        # ensure directory doesn't exist
        try:
            rmtree(SEARCH_DIR)
        except:
            pass
        makedirs(SEARCH_DIR)

    def tearDown(self):
        """This method is run once after _each_ test method is executed"""
        # Eliminate any variables defined
        del os.environ['%sTEMPDIR' % SAB_ENVIRO_ID]
        if '%sVERSION' % SAB_ENVIRO_ID in os.environ:
            del os.environ['%sVERSION' % SAB_ENVIRO_ID]
        del os.environ['%sDIRECTORY' % SAB_ENVIRO_ID]
        del os.environ['%sNZBNAME' % SAB_ENVIRO_ID]
        if '%sCOMPLETED_DIR' % SAB_ENVIRO_ID in os.environ:
            del os.environ['%sCOMPLETED_DIR' % SAB_ENVIRO_ID]
        del os.environ['%sCAT' % SAB_ENVIRO_ID]
        del os.environ['%sPP_STATUS' % SAB_ENVIRO_ID]

        try:
            rmtree(SEARCH_DIR)
        except:
            pass

        # common
        super(TestSABPostProcessScript, self).tearDown()

    def test_main_returns(self):
        # a NZB Logger set to False uses stderr
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == SHELL_EXIT_CODE.SUCCESS

        class TestSABPostProcessMain(SABPostProcessScript):
            def sabnzbd_postprocess_main(self, *args, **kwargs):
                return None

        script = TestSABPostProcessMain(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == SHELL_EXIT_CODE.NONE

        del script
        # However we always pass in we validate our environment correctly
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == SHELL_EXIT_CODE.SUCCESS

        # If we fail to validate, then we flat out fail
        del os.environ['%sVERSION' % SAB_ENVIRO_ID]
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.run() == SHELL_EXIT_CODE.FAILURE

    def test_environment_varable_init(self):
        """
        Testing NZBGet Script initialization using environment variables
        """
        # a NZB Logger set to False uses stderr
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.directory == DIRECTORY
        assert script.nzbname == NZBNAME
        assert script.nzbfilename == NZBFILENAME
        assert script.category == CATEGORY

        assert script.system['TEMPDIR'] == TEMP_DIRECTORY
        assert script.system['VERSION'] == str(VERSION)
        assert script.system['DIRECTORY'] == DIRECTORY
        assert script.system['COMPLETE_DIR'] == COMPLETE_DIRECTORY
        assert script.system['FILENAME'] == NZBNAME
        assert script.system['NZBNAME'] == NZBFILENAME
        assert script.system['CAT'] == CATEGORY
        assert script.system['PP_STATUS'] == STATUS

        assert script.get('TEMPDIR') == TEMP_DIRECTORY
        assert script.get('VERSION') == str(VERSION)
        assert script.get('DIRECTORY') == DIRECTORY
        assert script.get('COMPLETE_DIR') == COMPLETE_DIRECTORY
        assert script.get('FILENAME') == NZBNAME
        assert script.get('NZBNAME') == NZBFILENAME
        assert script.get('CAT') == CATEGORY
        assert script.get('PP_STATUS') == STATUS

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG

        assert script.nzbheaders['MOVIEYEAR'] == '1983'
        assert script.nzbheaders['NAME'] == \
            'A.Great.Movie.1983.DVDRip.x264-AWESOME'
        assert script.nzbheaders['PROPERNAME'] == 'A Great Movie'
        assert script.nzbheaders['CATEGORY'] == 'Movies > SD'

        assert script.nzb_get('movieyear') == '1983'
        assert script.nzb_get('name') == \
            'A.Great.Movie.1983.DVDRip.x264-AWESOME'
        assert script.nzb_get('propername') == 'A Great Movie'
        assert script.nzb_get('category') == 'Movies > SD'

        assert os.environ['%sTEMPDIR' % SAB_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sDIRECTORY' % SAB_ENVIRO_ID] == DIRECTORY
        assert os.environ['%sCOMPLETE_DIR' % SAB_ENVIRO_ID] == COMPLETE_DIRECTORY
        assert os.environ['%sFILENAME' % SAB_ENVIRO_ID] == NZBNAME
        assert os.environ['%sNZBNAME' % SAB_ENVIRO_ID] == NZBFILENAME
        assert os.environ['%sCAT' % SAB_ENVIRO_ID] == CATEGORY
        assert os.environ['%sPP_STATUS' % SAB_ENVIRO_ID] == STATUS

    def test_environment_override(self):
        """
        Testing NZBGet Script initialization using forced variables that
        should take priority over global ones
        """
        directory = join(DIRECTORY, 'a', 'deeper', 'path')
        # ensure directory doesn't exist
        try:
            rmtree(directory)
        except:
            pass
        makedirs(directory)

        nzbname = '%s.with.more.content' % NZBNAME
        nzbfilename = join(directory, basename(NZBFILENAME))
        category = '%s2' % CATEGORY
        status = str(PP_STATUS.FAILURE)

        script = SABPostProcessScript(
            logger=False,
            debug=VERY_VERBOSE_DEBUG,

            directory=directory,
            nzbname=nzbname,
            nzbfilename=nzbfilename,
            category=category,
            status=status,
        )

        assert script.directory == directory
        assert script.nzbname == nzbname
        assert script.nzbfilename == nzbfilename
        assert script.category == category
        assert str(script.status) == status

        assert script.system['TEMPDIR'] == TEMP_DIRECTORY
        assert script.system['DIRECTORY'] == directory
        assert script.system['FILENAME'] == nzbname
        assert script.system['NZBNAME'] == nzbfilename
        assert script.system['CAT'] == category
        assert script.system['PP_STATUS'] == status

        assert len(script.config) == 1
        assert script.config.get('DEBUG') == VERY_VERBOSE_DEBUG
        assert script.shared == {}
        assert script.nzbheaders == {}

        assert os.environ['%sTEMPDIR' % SAB_ENVIRO_ID] == TEMP_DIRECTORY
        assert os.environ['%sDIRECTORY' % SAB_ENVIRO_ID] == directory
        assert os.environ['%sFILENAME' % SAB_ENVIRO_ID] == nzbname
        assert os.environ['%sNZBNAME' % SAB_ENVIRO_ID] == nzbfilename
        assert os.environ['%sCAT' % SAB_ENVIRO_ID] == category
        assert os.environ['%sPP_STATUS' % SAB_ENVIRO_ID] == status

        # cleanup
        try:
            rmtree(directory)
        except:
            pass

    def test_validation(self):

        # This will fail because it's looking for the VERSION
        # variable defined with SABnzbd v2.1
        # a NZB Logger set to False uses stderr
        if '%sVERSION' % SAB_ENVIRO_ID in os.environ:
            del os.environ['%sVERSION' % SAB_ENVIRO_ID]
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert not script.validate()
        del script

        # Now let's set it and try again
        os.environ['%sVERSION' % SAB_ENVIRO_ID] = VERSION
        # a NZB Logger set to False uses stderr
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.validate()
        del script

        # Now we'll try arguments
        os.environ['%sVALUE_A' % SAB_ENVIRO_ID] = 'A'
        os.environ['%sVALUE_B' % SAB_ENVIRO_ID] = 'B'
        os.environ['%sVALUE_C' % SAB_ENVIRO_ID] = 'C'

        # a NZB Logger set to False uses stderr
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)
        assert script.validate(keys=(
            'Value_A', 'VALUE_B', 'value_c'
        ))
        del os.environ['%sVALUE_A' % SAB_ENVIRO_ID]
        del os.environ['%sVALUE_B' % SAB_ENVIRO_ID]
        del os.environ['%sVALUE_C' % SAB_ENVIRO_ID]

    def test_gzipped_nzbfiles(self):
        """
        Test gziped nzbfiles
        """
        os.environ['%sORIG_NZB_GZ' % SAB_ENVIRO_ID] = join(
            dirname(__file__), 'var', 'plain.nzb.gz')

        # Create our object
        script = SABPostProcessScript(logger=False, debug=VERY_VERBOSE_DEBUG)

        # We have a temporary file now
        tmp_file = script._sab_temp_nzb
        assert(isfile(tmp_file) is True)

        # Running our script will just return zero
        assert(script.run() == 0)

        # But not after we exit, our tmp_file would have been cleaned up
        assert(isfile(tmp_file) is False)

        # We should be able to uncompress this file

        del os.environ['%sORIG_NZB_GZ' % SAB_ENVIRO_ID]



# -*- encoding: utf-8 -*-
#
# simplify importing of comonly used modules
#
# Copyright (C) 2014-2017 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
__title__ = 'pynzbget'
__version__ = '0.6.2'
__author__ = 'Chris Caron <lead2gold@gmail.com>'
__license__ = 'GPLv3'
__copywrite__ = 'Copyright 2014-2017 Chris Caron <lead2gold@gmail.com>'

from .ScriptBase import *
from .SchedulerScript import *
from .PostProcessScript import *
from .SABPostProcessScript import *
from .FeedScript import *
from .ScanScript import *
from .QueueScript import *



# -*- encoding: utf-8 -*-
#
# Some common utilities that may prove useful when processing downloads
#
# Copyright (C) 2014 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
import re
from os.path import expanduser

try:
    from xml.sax.saxutils import unescape
    SAX_UNESCAPE = True
except ImportError:
    SAX_UNESCAPE = False
    from HTMLParser import HTMLParser

# Pre-Escape content since we reference it so much
ESCAPED_PATH_SEPARATOR = re.escape('\\/')
ESCAPED_WIN_PATH_SEPARATOR = re.escape('\\')
ESCAPED_NUX_PATH_SEPARATOR = re.escape('/')

TIDY_WIN_PATH_RE = re.compile(
    '(^[%s]{2}|[^%s\s][%s]|[\s][%s]{2}])([%s]+)' % (
        ESCAPED_WIN_PATH_SEPARATOR,
        ESCAPED_WIN_PATH_SEPARATOR,
        ESCAPED_WIN_PATH_SEPARATOR,
        ESCAPED_WIN_PATH_SEPARATOR,
        ESCAPED_WIN_PATH_SEPARATOR,
))
TIDY_WIN_TRIM_RE = re.compile(
    '^(.+[^:][^%s])[\s%s]*$' %(
        ESCAPED_WIN_PATH_SEPARATOR,
        ESCAPED_WIN_PATH_SEPARATOR,
))

TIDY_NUX_PATH_RE = re.compile(
    '([%s])([%s]+)' % (
        ESCAPED_NUX_PATH_SEPARATOR,
        ESCAPED_NUX_PATH_SEPARATOR,
))
TIDY_NUX_TRIM_RE = re.compile(
    '([^%s])[\s%s]+$' % (
        ESCAPED_NUX_PATH_SEPARATOR,
        ESCAPED_NUX_PATH_SEPARATOR,
))

def os_path_split(path):
    """splits a path into a list by it's path delimiter

       hence: split_path('/etc/test/file') outputs:
                ['', 'etc', 'test', 'file']

       relative paths don't have the blank entry at the head
       of the string:
       hence: split_path('relative/file') outputs:
                ['relative', 'file']

       Paths can be reassembed as follows:
       assert '/'.join(split_path('/etc/test/file')) == \
               '/etc/test/file'

       assert '/'.join(split_path('relative/file')) == \
               'relative/file'
    """
    path = path.strip()
    if not path:
        return []

    p_list = re.split('[%s]+' % ESCAPED_PATH_SEPARATOR, path)
    try:
        # remove trailing slashes
        while not p_list[-1] and len(p_list) > 1:
            p_list.pop()
    except IndexError:
        # Nothing passed in
        return []
    return p_list

def tidy_path(path):
    """take a filename and or directory and attempts to tidy it up by removing
    trailing slashes and correcting any formatting issues.

    For example: ////absolute//path// becomes:
        /absolute/path

    """
    # Windows
    path = TIDY_WIN_PATH_RE.sub('\\1', path.strip())
    # Linux
    path = TIDY_NUX_PATH_RE.sub('\\1', path.strip())

    # Linux Based Trim
    path = TIDY_NUX_TRIM_RE.sub('\\1', path.strip())
    # Windows Based Trim
    path = expanduser(TIDY_WIN_TRIM_RE.sub('\\1', path.strip()))
    return path

def unescape_xml(content):
    """
    Escapes XML content into it's regular string value
    """

    if SAX_UNESCAPE:
        return unescape(content)
    return HTMLParser().unescape(content)



# -*- encoding: utf-8 -*-
#
# A scripting wrapper for NZBGet's Post Processing Scripting
#
# Copyright (C) 2014 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
"""
This class was intended to make writing NZBGet Scripts easier to manage and
write by handling the common error handling and provide the most reused code
in a re-usable container. It was initially written to work with NZBGet v13
but provides most backwards compatibility.

It was designed to be inheritied as a base class requiring you to only write
the main() function which should preform the task you are intending.

It looks after fetching all of the environment variables and will parse
the meta information out of the NZB-File.

It allows you to set variables that other scripts can access if they need to
using the set() and get() variables. This is done through a simply self
maintained hash table type structure within a sqlite database. All the
wrapper functions are already written.  If you call 'set('MYKEY', 1')
you can call get('MYKEY') in another script and continue working

push() functions written to pass information back to nzbget using it's
processing engine.

all exceptions are now automatically handled and logging can be easily
changed from stdout, to stderr or to a file.

Test suite built in (using python-nose) to ensure old global variables
will still work as well as make them easier to access and manipulate.

Some inline documentation was based on content provided at:
   - http://nzbget.net/Extension_scripts


############################################################################
Post Process Script Usage/Example
############################################################################

#############################################################################
### NZBGET POST-PROCESSING SCRIPT                                         ###
#
# Describe your Post-Process Script here
# Author: Chris Caron <lead2gold@gmail.com>
#

############################################################################
### OPTIONS                                                              ###

#
# Enable NZBGet debug logging (yes, no)
# Debug=no
#

### NZBGET POST-PROCESSING SCRIPT                                         ###
#############################################################################

from nzbget import PostProcessScript

# Now define your class while inheriting the rest
class MyPostProcessScript(PostProcessScript):
    def main(self, *args, **kwargs):

        # Version Checking, Environment Variables Present, etc
        if not self.validate():
            # No need to document a failure, validate will do that
            # on the reason it failed anyway
            return False

        # write all of your code here you would have otherwise put in the
        # script

        # All system environment variables (NZBOP_.*) as well as Post
        # Process script specific content (NZBPP_.*)
        # following dictionary (without the NZBOP_ or NZBPP_ prefix):
        print('TEMPDIR (directory is: %s' % self.get('TEMPDIR'))
        print('DIRECTORY %s' self.get('DIRECTORY'))
        print('NZBNAME %s' self.get('NZBNAME'))
        print('NZBFILENAME %s' self.get('NZBFILENAME'))
        print('CATEGORY %s' self.get('CATEGORY'))
        print('TOTALSTATUS %s' self.get('TOTALSTATUS'))
        print('STATUS %s' self.get('STATUS'))
        print('SCRIPTSTATUS %s' self.get('SCRIPTSTATUS'))

        # Set any variable you want by any key.  Note that if you use
        # keys that were defined by the system (such as CATEGORY, DIRECTORY,
        # etc, you may have some undesirable results.  Try to avoid reusing
        # system variables already defined (identified above):
        self.set('MY_KEY', 'MY_VALUE')

        # You can fetch it back; this will also set an entry in  the
        # sqlite database for each hash references that can be pulled from
        # another script that simply calls self.get('MY_KEY')
        print(self.get('MY_KEY')) # prints MY_VALUE

        # You can also use push() which is similar to set()
        # except that it interacts with the NZBGet Server and does not use
        # the sqlite database. This can only be reached across other
        # scripts if the calling application is NZBGet itself
        self.push('ANOTHER_KEY', 'ANOTHER_VALUE')

        # You can still however locally retrieve what you set using push()
        # with the get() function
        print(self.get('ANOTHER_KEY')) # prints ANOTHER_VALUE

        # Your script configuration files (NZBPP_.*) are here in this
        # dictionary (again without the NZBPP_ prefix):
        # assume you defined `Debug=no` in the first 10K of your PostProcessScript
        # NZBGet translates this to `NZBPP_DEBUG` which can be retrieved
        # as follows:
        print('DEBUG %s' self.get('DEBUG'))

        # Returns have been made easy.  Just return:
        #   * True if everything was successful
        #   * False if there was a problem
        #   * None if you want to report that you've just gracefully
                  skipped processing (this is better then False)
                  in some circumstances. This is neither a failure or a
                  success status.

        # Feel free to use the actual exit codes as well defined by
        # NZBGet on their website.  They have also been defined here
        # from nzbget import EXIT_CODE

        return True

# Call your script as follows:
if __name__ == "__main__":
    from sys import exit

    # Create an instance of your Script
    myscript = MyPostProcessScript()

    # call run() and exit() using it's returned value
    exit(myscript.run())
"""

import re
from os import chdir
from os import environ
from os.path import isdir
from os.path import isfile
from os.path import join
from os.path import splitext
from os.path import basename
from os.path import dirname
from os.path import abspath

from socket import error as SocketError

# Relative Includes
from .ScriptBase import ScriptBase
from .ScriptBase import Health
from .ScriptBase import SCRIPT_MODE
from .ScriptBase import NZBGET_BOOL_FALSE
from .Utils import os_path_split as split

from .PostProcessCommon import OBFUSCATED_PATH_RE
from .PostProcessCommon import OBFUSCATED_FILE_RE
from .PostProcessCommon import SCRIPT_STATUS
from .PostProcessCommon import PAR_STATUS
from .PostProcessCommon import UNPACK_STATUS


class TOTAL_STATUS(object):
    """Cumulative (Total) Status of NZB Processing
    """
    # everything OK
    SUCCESS = 'SUCCESS'
    # download is damaged but probably can be repaired; user intervention is
    # required;
    WARNING = 'WARNING'
    # download has failed or a serious error occurred during
    # post-processing (unpack, par);
    FAILURE = 'FAILURE'
    # download was deleted; post-processing scripts are usually not called in
    # this case; however it's possible to force calling scripts with command
    # "post-process again".
    DELETED = 'DELETED'

# Environment variable that prefixes all NZBGET options being passed into
# scripts with respect to the NZB-File (used in Post Processing Scripts)
POSTPROC_ENVIRO_ID = 'NZBPP_'

# Precompile Regulare Expression for Speed
POSTPROC_OPTS_RE = re.compile('^%s([A-Z0-9_]+)$' % POSTPROC_ENVIRO_ID)

class PostProcessScript(ScriptBase):
    """POST PROCESS mode is called after the unpack stage
    """
    def __init__(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Multi-Script Support
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if not hasattr(self, 'script_dict'):
            # Only define once
            self.script_dict = {}
        self.script_dict[SCRIPT_MODE.POSTPROCESSING] = self

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Initialize Parent
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        super(PostProcessScript, self).__init__(*args, **kwargs)

    def postprocess_init(self, *args, **kwargs):
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Fetch Script Specific Arguments
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        directory = kwargs.get('directory')
        nzbname = kwargs.get('nzbname')
        nzbfilename = kwargs.get('nzbfilename')
        category = kwargs.get('category')
        totalstatus = kwargs.get('totalstatus')
        status = kwargs.get('status')
        scriptstatus = kwargs.get('scriptstatus')
        parse_nzbfile = kwargs.get('parse_nzbfile', True)
        use_database = kwargs.get('use_database', True)

        # Support Depricated Variables
        parstatus = kwargs.get('parstatus')
        unpackstatus = kwargs.get('unpackstatus')

        # Fetch/Load Post Process Script Configuration
        script_config = dict([(POSTPROC_OPTS_RE.match(k).group(1), v.strip()) \
               for (k, v) in environ.items() if POSTPROC_OPTS_RE.match(k)])

        if self.vvdebug:
            # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
            # Print Global Script Varables to help debugging process
            # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
            for k, v in script_config.items():
                self.logger.vvdebug('%s%s=%s' % (POSTPROC_ENVIRO_ID, k, v))

        # Merge Script Configuration With System Config
        self.system = dict(script_config.items() + self.system.items())

        # self.directory
        # This is the path to the destination directory for downloaded files.
        if directory is None:
            self.directory = environ.get(
                '%sDIRECTORY' % POSTPROC_ENVIRO_ID,
            )
            _final_directory = environ.get(
                '%sFINALDIR' % POSTPROC_ENVIRO_ID,
            )
            if self.directory and not isdir(self.directory):
                if _final_directory and isdir(_final_directory):
                    # adjust path
                    self.directory = _final_directory
        else:
            self.directory = directory

        if self.directory:
            self.directory = abspath(self.directory)

        # self.nzbname
        # User-friendly name of processed nzb-file as it is displayed by the
        # program.  The file path and extension are removed.  If download was
        # renamed, this parameter reflects the new name.
        if nzbname is None:
            self.nzbname = environ.get(
                '%sNZBNAME' % POSTPROC_ENVIRO_ID,
            )
        else:
            self.nzbname = nzbname

        # self.nzbfilename
        # Name of processed nzb-file. If the file was added from incoming
        # nzb-directory, this is a full file name, including path and
        # extension. If the file was added from web-interface, it's only the
        # file name with extension. If the file was added via RPC-API (method
        # append), this can be any string but the use of actual file name is
        # recommended for developers.
        if nzbfilename is None:
            self.nzbfilename = environ.get(
                '%sNZBFILENAME' % POSTPROC_ENVIRO_ID,
            )
        else:
            self.nzbfilename = nzbfilename

        # self.category
        # Category assigned to nzb-file (can be empty string).
        if category is None:
            self.category = environ.get(
                '%sCATEGORY' % POSTPROC_ENVIRO_ID,
            )
        else:
            self.category = category

        # self.totalstatus
        # Total status of the processing of the NZB-File.  This value
        # includes the result from previous scripts that may have ran
        # before this one.
        if totalstatus is None:
            self.totalstatus = environ.get(
                '%sTOTALSTATUS' % POSTPROC_ENVIRO_ID,
            )
        else:
            self.totalstatus = totalstatus

        # self.status
        # Complete status info for nzb-file: it consists of total status and
        # status detail separated with slash. There are many combinations.
        # Just few examples:
        #         FAILURE/HEALTH
        #         FAILURE/PAR
        #         FAILURE/UNPACK
        #         WARNING/REPAIRABLE
        #         WARNING/SPACE
        #         WARNING/PASSWORD
        #         SUCCESS/ALL
        #         SUCCESS/UNPACK
        #
        # For the complete list see description of method history in RPC API
        # reference: http://nzbget.net/RPC_API_reference
        if status is None:
            self.status = Health(environ.get(
                '%sSTATUS' % POSTPROC_ENVIRO_ID,
            ))
        else:
            self.status = Health(status)

        # self.scriptstatus
        # Summary status of the scripts executed before the current one
        if scriptstatus is None:
            self.scriptstatus = environ.get(
                '%sSCRIPTSTATUS' % POSTPROC_ENVIRO_ID,
            )
        else:
            self.scriptstatus = scriptstatus

        # self.parstatus (NZBGet < v13) - Depreciated
        # Result of par-check
        if parstatus is None:
            self.parstatus = environ.get(
                '%sPARSTATUS' % POSTPROC_ENVIRO_ID,
                # Default
                PAR_STATUS.SKIPPED,
            )
        else:
            self.parstatus = parstatus

        # self.unpackstatus (NZBGet < v13) - Depreciated
        # Result of unpack
        if unpackstatus is None:
            self.unpackstatus = environ.get(
                '%sUNPACKSTATUS' % POSTPROC_ENVIRO_ID,
                # Default
                UNPACK_STATUS.SKIPPED,
            )
        else:
            self.unpackstatus = unpackstatus

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Error Handling
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if self.nzbfilename:
            # absolute path names
            self.nzbfilename = abspath(self.nzbfilename)

            if parse_nzbfile:
                # Initialize information fetched from NZB-File
                # We intentionally allow existing nzbheaders to over-ride
                # any found in the nzbfile
                self.nzbheaders = dict(
                    self.parse_nzbfile(
                        self.nzbfilename, check_queued=True)\
                        .items() + self.pull_dnzb().items(),
                )

        if self.directory:
            # absolute path names
            self.directory = abspath(self.directory)

        if not (self.directory and isdir(self.directory)):
            self.logger.debug('Process directory is missing: %s' % \
                self.directory)
        else:
            try:
                chdir(self.directory)
            except OSError:
                self.logger.debug('Process directory is not accessible: %s' % \
                    self.directory)

        # Total Status
        if not isinstance(self.totalstatus, basestring):
            self.totalstatus = TOTAL_STATUS.SUCCESS

        # Par Status
        if not isinstance(self.parstatus, int):
            try:
                self.parstatus = int(self.parstatus)
            except:
                self.parstatus = PAR_STATUS.SKIPPED

        # Unpack Status
        if not isinstance(self.unpackstatus, int):
            try:
                self.unpackstatus = int(self.unpackstatus)
            except:
                self.unpackstatus = UNPACK_STATUS.SKIPPED

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Enforce system/global variables for script processing
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        self.system['DIRECTORY'] = self.directory
        if self.directory is not None:
            environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = \
                self.directory

        self.system['NZBNAME'] = self.nzbname
        if self.nzbname is not None:
            environ['%sNZBNAME' % POSTPROC_ENVIRO_ID] = \
                self.nzbname

        self.system['NZBFILENAME'] = self.nzbfilename
        if self.nzbfilename is not None:
            environ['%sNZBFILENAME' % POSTPROC_ENVIRO_ID] = \
                self.nzbfilename

        self.system['CATEGORY'] = self.category
        if self.category is not None:
            environ['%sCATEGORY' % POSTPROC_ENVIRO_ID] = \
                self.category

        self.system['TOTALSTATUS'] = self.totalstatus
        if self.totalstatus is not None:
            environ['%sTOTALSTATUS' % POSTPROC_ENVIRO_ID] = \
                self.totalstatus

        self.system['STATUS'] = str(self.status)
        if self.status is not None:
            environ['%sSTATUS' % POSTPROC_ENVIRO_ID] = \
                str(self.status)

        self.system['SCRIPTSTATUS'] = self.scriptstatus
        if self.scriptstatus is not None:
            environ['%sSCRIPTSTATUS' % POSTPROC_ENVIRO_ID] = \
                self.scriptstatus

        self.system['PARSTATUS'] = self.parstatus
        if self.parstatus is not None:
            environ['%sPARSTATUS' % POSTPROC_ENVIRO_ID] = \
                str(self.parstatus)

        self.system['UNPACKSTATUS'] = self.unpackstatus
        if self.unpackstatus is not None:
            environ['%sUNPACKSTATUS' % POSTPROC_ENVIRO_ID] = \
                str(self.unpackstatus)

        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        # Create Database for set() and get() operations
        # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
        if use_database:
            # database_key is inherited in the parent class
            # future calls of set() and get() will allow access
            # to the database now
            try:
                self.database_key = \
                        self.get('NZBID', basename(self.nzbfilename))
            except AttributeError:
                pass

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Debug Flag Check
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def postprocess_debug(self, *args, **kwargs):
        """Uses the environment variables to detect if debug mode is set
        """
        return self.parse_bool(
            environ.get('%sDEBUG' % POSTPROC_ENVIRO_ID, NZBGET_BOOL_FALSE),
        )

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Sanity
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def postprocess_sanity_check(self, *args, **kwargs):
        """Sanity checking to ensure this really is a Post-Process Script
        """
        return ('%sDIRECTORY' % POSTPROC_ENVIRO_ID in environ)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Validatation
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def postprocess_validate(self, keys=None, min_version=11, *args, **kwargs):
        """validate against environment variables
        """
        is_okay = super(PostProcessScript, self)._validate(
            keys=keys,
            min_version=min_version,
            download_okay=True,
        )

        if min_version >= 13:
            required_opts = set((
                'TOTALSTATUS',
                'STATUS',
                'SCRIPTSTATUS',
            ))
            found_opts = set(self.system) & required_opts
            if found_opts != required_opts:
                missing_opts = list(required_opts ^ found_opts)
                self.logger.error(
                    'Validation - (v13) Directives not set: %s' % \
                      ', '.join(missing_opts),
                )
                is_okay = False

        return is_okay

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Health Check
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def postprocess_health_check(self, is_unpacked=True, has_archive=False,
                                 *args, **kwargs):
        """Similar to validate, except some scripts don't need to
        out right fail if the download health is bad. Some might just
        return silently, others may try to correct the health
        """

        is_okay = super(PostProcessScript, self)\
                ._health_check(*args, **kwargs)

        if self.version < 13:
            # We need to be sure the download is okay before continuing
            # using depricated values in newer version
            if self.parstatus == PAR_STATUS.FAILURE:
                if is_unpacked:
                    is_okay = False

            if self.unpackstatus == UNPACK_STATUS.FAILURE:
                if is_unpacked:
                    is_okay = False

        else:
            if is_unpacked and not self.status.is_unpacked:
                is_okay = False

            if has_archive and not self.status.has_archive:
                is_okay = False

        return is_okay

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # File Retrieval
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def postprocess_get_files(self, search_dir=None, *args, **kwargs):
        """a wrapper to the get_files() function defined in the inherited class
           the only difference is the search_dir automatically uses the
           defined download `directory` as a default (if not specified).
        """
        if search_dir is None:
            search_dir = self.directory

        return super(PostProcessScript, self)._get_files(
            search_dir=search_dir, *args, **kwargs)

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Retrieve Statistics
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def get_statistics(self, nzbid=None):
        """
        Returns the download statistics (via the API)

        The result is returned in an easy to interpret dictionary like so
        {
            'download_size_mb': 10.00,
            'download_time_sec': 40.0,
            'download_avg': 2.0,
            'download_avg_unit': 'GB/s',

            'par_prepare_time_sec': 10.00,
            'par_repair_time_sec': 20.00,
            'unpack_time_sec': 30.00,
            'postprocess_time': 120.00,
            'par_total_time_sec': 30.00,
            'total_time_sec': 160.00,
        }

        If an error occurs, then 'None' is returned.
        """

        if not self.api_connect():
            return None

        if nzbid is None:
            try:
                nzbid = int(self.get('nzbID'))
            except (AttributeError, TypeError):
                return None

        try:
            group = [ x for x in self.api.listgroups(0) \
                     if x['NZBID'] == nzbid ][0]

        except SocketError as e:
            self.logger.warning('RCP Connection Failure (%d): %s' % (
                e[0], e[1]))
            return None

        except IndexError:
            return None

        try:
            dl_size = float(group['DownloadedSizeMB'])
            dl_time = float(group['DownloadTimeSec'])

        except KeyError:
            # We simply didn't fetch any valid content
            self.logger.warning('RCP Response was invalid.')
            self.logger.debug('RCP Response: %s', str(group))
            return None

        # Calculate statistics based on retrieved results
        dl_avg_speed = 0
        dl_avg_speed_unit = 'MB/s'

        if dl_time > 0.0:
            dl_avg_speed = (dl_size/dl_time)
            if dl_avg_speed < 1.0:
                # Convert to KB/s
                dl_avg_speed_unit = 'KB/s'
                dl_avg_speed *= 1024.0
            elif dl_avg_speed > 1000.0:
                # Convert to GB/s
                dl_avg_speed_unit = 'GB/s'
                dl_avg_speed /= 1024.0

        unpack_time = float(group['UnpackTimeSec'])
        par_prepare_time = float(group['ParTimeSec'])
        par_repair_time = float(group['RepairTimeSec'])
        postprocess_time = float(group['PostTotalTimeSec'])

        return {
            # File Download Size (in MB)
            'download_size_mb': dl_size,
            # File Download Total Time (in seconds)
            'download_time_sec': dl_time,
            # File Download Average Transfer Speed
            'download_avg': dl_avg_speed,
            # File Download Average Transfer Speed Unit
            'download_avg_unit': dl_avg_speed_unit,

            # Time Taken to check integrity of file (in seconds)
            'par_prepare_time_sec': par_prepare_time,
            # Time Taken to repair file if it was damaged (in seconds)
            'par_repair_time_sec': par_repair_time,
            # Time Taken to unpack file from it's compressed archive(s)
            # (in seconds)
            'unpack_time_sec': unpack_time,
            # General time spent post processing download (in seconds)
            'postprocess_time': postprocess_time,
            # General time spent verifying and repairing download (in seconds)
            'par_total_time_sec': par_prepare_time + par_repair_time,
            # Total estimated system time spent handling this download
            'total_time_sec': postprocess_time + float(group['DownloadTimeSec']),
        }

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Obfuscation Handling
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def deobfuscate(self, filename, ref_dir=None, ref_nzbfile=None):
        """attempts to detect and update
        """

        if ref_dir is None:
            ref_dir = self.directory

        if ref_nzbfile is None:
            ref_nzbfile = self.nzbfilename

        if filename[0:len(ref_dir):] == ref_dir:
            new_name = filename[len(ref_dir)+1:]
            self.logger.debug('Deobfuscate - Stripped filename down to: %s' % new_name)
        else:
            new_name = filename

        parts = split(new_name)
        part_removed = 0
        for x in range(0, len(parts)-1):
            fn = parts[x]
            if OBFUSCATED_PATH_RE.match(fn):
                self.logger.info(
                    'Detected obfuscated directory name %s,' % fn + \
                    ' removing from path',
                )
                parts[x] = None
                part_removed += 1

        if OBFUSCATED_FILE_RE.match(basename(filename)):
            self.logger.info(
                'Detected obfuscated filename %s,' % basename(filename) + \
                ' removing from path')
            parts[len(parts)-1] = '-' + splitext(filename)[1]
            part_removed += 1

        if part_removed < len(parts):
            new_name = ''
            for x in range(0, len(parts)):
                if parts[x] != None:
                    new_name = join(new_name, parts[x])
            return new_name

        # If we reach here, we have a completely deobfuscated file

        # Check out NZB-Filename
        if len(self.nzb_items()):
            self.logger.info(
                'All file path parts are obfuscated, using obfuscated ' + \
                'NZB-Headers',
            )

            # Fetch category
            category = self.nzb_get('category', '')\
                .split(' ')[0].lower()
            subcategory = self.nzb_get('category', '')\
                .split(' ')[-1].lower()

            if self.nzb_get('name'):
                # We can pick from the nzb headers
                nzb_name = self.nzb_get('name')
                new_name = join(ref_dir, '%s%s' %(
                    re.sub('[\s]+','.', nzb_name),
                    splitext(basename(new_name))[1],
                ))

            elif category[0:5] == 'movie' and \
                    self.nzb_get('propername'):
                nzb_name = self.nzb_get('propername')

                if self.nzb_get('movieyear'):
                    nzb_name += '(%s)' % self.nzb_get('movieyear')

                new_name = join(ref_dir, '%s%s' %(
                    re.sub('[\s]+','.', nzb_name),
                    splitext(basename(new_name))[1],
                ))

            elif category == 'tv' and \
                    self.nzb_get('propername'):
                nzb_name = self.nzb_get('propername')
                if self.nzb_get('episodename'):
                    nzb_name += '-%s' % \
                            self.nzb_get('episodename')

                if subcategory == 'hd':
                    nzb_name += '-HDTV'

                new_name = join(ref_dir, '%s%s' %(
                    re.sub('[\s]+','.', nzb_name),
                    splitext(basename(new_name))[1],
                ))

            elif self.nzb_get('propername'):
                nzb_name = self.nzb_get('propername')
                new_name = join(ref_dir, '%s%s' %(
                    re.sub('[\s]+','.', nzb_name),
                    splitext(basename(filename))[1],
                ))
            else:
                # No possible
                new_name = ''

        if new_name:
            self.logger.debug('Deobfuscate - Generated filename: %s' % new_name)
            return new_name

        # we're running out of new names :)... try the NZB-FileName
        if ref_nzbfile and not OBFUSCATED_FILE_RE.match(basename(ref_nzbfile)):
            self.logger.info(
                'All file path parts are obfuscated, using NZB-FileName',
            )
            new_name = join(ref_dir, '%s%s' %(
                re.sub('[\s]+','.',
                       splitext(basename(ref_nzbfile))[0]),
                splitext(basename(filename))[1],
            ))

        else:
            self.logger.info('Deobfuscation is not possible')
            return filename

        self.logger.debug('Deobfuscate - Generated filename: %s' % new_name)
        return new_name

    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    # Set/Control Functions (also passes data back to NZBGet)
    # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    def push_directory(self, directory=None, final=False):
        if directory:
            # Update local directory
            self.directory = directory

            # Accomodate other environmental variables
            self.system['DIRECTORY'] = self.directory
            environ['%sDIRECTORY' % POSTPROC_ENVIRO_ID] = self.directory

        # Alert NZBGet of Change
        if not final:
            key = 'DIRECTORY'
        else:
            key = 'FINALDIR'

        return self._push(key=key, value=self.directory)



# -*- encoding: utf-8 -*-
#
# A simple SQLite (3) Database wrapper for shared NZBGet transactions
#
# Copyright (C) 2014 Chris Caron <lead2gold@gmail.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
"""
This script will allow a common gateway between scripts that require
interaction with one another. It sets up a simple hash table like system
that can be used as a method of passing variables around using the
get() and set() operation.

It's initialized by setting up a container object. This keeps all set()
get() operations limited to what is set by the container itself.

NZBScripts should not be dependant on one another.  The intention
is not to use this to set a variable so it can be retrieved late by another
script. However setting a variable in advance that another script needs
to preform the same proceedure to get to can be grealy simplified by sharing
cpu power by just sharing your results in advance.
"""
import sqlite3
import re
from datetime import datetime
from datetime import timedelta
from os.path import isfile
from os.path import dirname
from os.path import isdir
from os import unlink
from os import makedirs

from .Logger import init_logger
from .Logger import destroy_logger
from logging import Logger

# This should always be set to the current database version
NZBGET_DATABASE_VERSION = 1

# In seconds, we identify how long to let content linger in the
# database for before it's purged (no sense letting content grow)
# below sets the timer for 12 hours.  This database contains just transient
# information; there is no need to keep content longer then this.
PURGE_AGE = 60 * 60 * 12

# Format required to correctly query and handle SQLite Dates
SQLITE_DATE_FORMAT = '%Y-%m-%d %H:%M:%S'

NZBGET_SCHEMA = {
  1: [
     # Lookup just contains static content and VSPs used by the database
     # and it's handling of data.  You should not write content here. this
     # is just reserved for internal operations
     "CREATE TABLE lookup (key TEXT PRIMARY KEY, value TEXT)",
     # Init base version of zero (0), the _build_schema takes care
     # of updating this value
     "INSERT INTO lookup (key, value) VALUES ('SCHEMA_VERSION', '0')",
     "INSERT INTO lookup (key, value) VALUES ('PURGE_AGE', '%d')" % \
         PURGE_AGE,
     # Key Store free for use for developers of scripts
     # just use the set() and get() functions
     "CREATE TABLE keystore (" + \
        "container TEXT, " + \
        "category TEXT, " + \
        "key TEXT, " + \
        "value TEXT, " + \
        "last_update DATETIME DEFAULT current_timestamp" + \
     ")",
     "CREATE UNIQUE INDEX keystore_idx ON keystore (container, category, key)",
     "CREATE INDEX last_update_idx ON keystore (last_update)",
  ],
}
# This is just used for a quick reference when verifying that
# all of the schema is present (during initialization)
# Define just table names here that were fully declared above in
# the schema
NZBGET_SCHEMA_TABLES = (
   u'lookup',
   u'keystore',
)

# Categories allow us to further partition our keystore hash table
# into groups for others subsections to use without stepping on
# each other.
class Category(object):
    # General Script Configuration
    CONFIG = u'config'
    # NZB/NZBD Defined Variables
    NZB = u'nzb'

CATEGORIES = [ Category.CONFIG, Category.NZB, ]
DEFAULT_CATEGORY = Category.CONFIG

# keys should not be complicated... make it so they aren't
VALID_KEY_RE = re.compile('[^a-zA-Z0-9_.-]')

class Database(object):
    def __init__(self, container, database, reset=False,
                 logger=True, debug=False):
        """Initializes the database if it isn't already prepared,
           Th en fetches an index to work with based on the key passed in.
           If reset is set to True, then if an existing entry is found, it is
           automatically reset and treated as a fresh process.
        """
        # self.container
        # This acts as the index for fetching content to and from
        # the keystore as well as tracking what goes on.
        self.container = container
        self.database = database

        # A global switch that basically disables this class silently
        # the purpose it to prevent it from thrashing on re-occuring
        # failures that are a result of the users system environment
        self.disabled = False

        # Database Connection
        self.socket = None

        # logger identifier
        self.logger_id = self.__class__.__name__
        self.logger = logger
        self.debug = debug

        if isinstance(self.logger, basestring):
            # Use Log File
            self.logger = init_logger(
                name=self.logger_id,
                logger=logger,
                debug=debug,
            )

        elif not isinstance(self.logger, Logger):
            # handle all other types
            if logger is None:
                # None means don't log anything
                self.logger = init_logger(
                    name=self.logger_id,
                    logger=None,
                    debug=debug,
                )
            else:
                # Use STDOUT for now
                self.logger = init_logger(
                    name=self.logger_id,
                    logger=True,
                    debug=debug,
                )
        else:
            self.logger_id = None

        if reset or not isfile(self.database):
            # Initialize
            self._reset()

        # Connect to Database
        if not self.connect():
            raise EnvironmentError('Could not access database.')

        if not self._schema_okay():
            # fail-safe
            self._reset()
            if not self._schema_okay():
                raise EnvironmentError('Could not prepare database.')

        # Keep content clean
        self.prune()

    def __del__(self):
        """Gracefully close any connection to the database on
           destruction of this class
        """
        self.close()
        if self.logger_id:
            destroy_logger(self.logger_id)

    def _reset(self, rebuild=True):
        """Resets the database
        If rebuild is set to True then the schema is re-prepared
        """
        try:
            self.close()
        except:
            pass

        try:
            # Best way to reset the database is to
            # remove it entirely
            unlink(self.database)
        except:
            pass

        if not isdir(dirname(self.database)):
            try:
                # safely ensure the directory exists
                makedirs(dirname(self.database))
            except:
                self.logger.error('Could not create directory: %s' % \
                                  dirname(self.database))
                return False

        if rebuild:
            # Connect to Database
            self.connect()
            if not self._schema_okay():
                if not self._build_schema():
                    return False

        self.logger.debug('Successsfully reset database: %s' % self.database)
        return True

    def connect(self):
        """Establish a connection to the database
        """
        if  self.socket is not None:
            # Already connected
            return True

        if self.disabled:
            # Connections turned off
            return False

        try:
            self.socket = sqlite3.connect(self.database, 20)
            self.logger.debug(
                'Opened connection to SQLite Database: %s' % \
                self.database,
            )

        except Exception as e:
            self.socket = None
            self.logger.error('Failed to connect SQLite Database')
            self.logger.debug(
                'SQLite Database (%s) failure message: %s' % (
                    self.database,
                    str(e),
            ))
            return False

        return True

    def close(self):
        if self.socket is not None:
            try:
                self.socket.close()
            except:
                pass

            self.socket = None
            self.logger.debug(
                'Closed connection to SQLite Database %s' % \
                self.database,
            )
        return

    def execute(self, *args, **kwargs):

        if not self.socket:
            if not self.connect():
                return None

        try:
            result = self.socket.execute(*args, **kwargs)
            self.logger.vdebug('DB Executing: %s' % str(args))

        except sqlite3.OperationalError as e:
            self.logger.debug('DB Execute OpError: %s' % str(e))
            return None

        except Exception as e:
            self.logger.debug('DB Execute Error: %s' % str(e))
            return None

        return result


    def _build_schema(self, start_version=None):
        """Build the schema or upgrade it (depending on the
           start_version defined)
        """
        if not self.socket:
            if not self.connect():
                return False

        if not isinstance(start_version, int):
            start_version = self._get_version()

        for version in [ k for k in sorted(NZBGET_SCHEMA.keys()) \
             if k > start_version]:
            for query in NZBGET_SCHEMA[version]:
                self.execute(query)
            self.execute(
                "UPDATE lookup SET value = ? " + \
                "WHERE key = 'SCHEMA_VERSION'",
                (str(version),),
            )
        return True

    def _schema_okay(self):
        """A simple check to see if the schema is okay
        """
        if not self.socket:
            if not self.connect():
                return False

        for table in NZBGET_SCHEMA_TABLES:
            try:
                if not bool(len(self.execute(
                    "SELECT 1 FROM sqlite_master WHERE name = ?", (table, )
                   ).fetchall())):
                    return False
            except AttributeError:
                # execute() returned None causing fetchall() not be
                # a valid attribute; this is the same as just being a
                # bad schema
                return False

        return self._get_version() == NZBGET_DATABASE_VERSION

    def _get_version(self):
        if not self.socket:
            if not self.connect():
                return 0

        result = self.execute(
            "SELECT value FROM lookup WHERE key = ?",
            ('SCHEMA_VERSION', ),
        )
        if result:
            try:
                return int(result.fetchall()[0][-1])
            except:
                return 0
        return 0

    def prune(self, age=None, vacuum=True):
        """Sweep old entries out of database to keep it's size
           under control
        """
        if not self.socket:
            if not self.connect():
                return False

        # Default
        prune_age = PURGE_AGE
        if not isinstance(age , int):
            result = self.execute(
                "SELECT value FROM lookup WHERE key = ?",
                ('PURGE_AGE', ),
            )
            if not result:
                # Simply put, if you remove this key, pruning will not
                # take place ever.
                return True

            try:
                prune_age = int(result.fetchall()[0][-1])
            except:
                pass
        else:
            prune_age = age

        purge_ref = datetime.now() - timedelta(seconds=prune_age)
        purge_ref = purge_ref.strftime(SQLITE_DATE_FORMAT)

        self.execute(
            "DELETE FROM keystore WHERE last_update <= ?",
            (purge_ref, ),
        )
        if vacuum:
            self.execute("VACUUM")

        return True

    def unset(self, key, category=None):
        """Remove a key from the database
        """
        if not self.socket:
            if not self.connect():
                return False

        if not category:
            category = DEFAULT_CATEGORY
        else:
            category = VALID_KEY_RE.sub('', category).lower()

        if category not in CATEGORIES:
            self.logger.error("Database category '%s' does not exist.")
            return False

        # clean key
        key = VALID_KEY_RE.sub('', key).upper()
        if not key:
            return False

        try:
            # First see if keystore already has a match
            if(bool(len(self.socket.execute(
                "SELECT value FROM keystore WHERE " + \
                "container = ? AND category = ? AND key = ?",
                (self.container, category, key)).fetchall()))):
                if not self.socket.execute(
                    "DELETE FROM keystore WHERE " +\
                    "container = ? AND category = ? AND key = ?",
                    (self.container, category, key),):
                    return False

        except sqlite3.OperationalError as e:
            # Database is corrupt or changed
            self.logger.debug(
                "Database.unset() Operational Error: %s" % str(e))

            if not self._reset():
                self.logger.error(
                    "Detected damaged database; countermeasures failed.",
                )
                self.disabled = True

            else:
                self.logger.info(
                    "Detected damaged database; situation corrected.",
                )

        return True


    def set(self, key, value, category=None):
        """Set a key and a value into the database for retrieval later
        """
        if not self.socket:
            if not self.connect():
                return False

        if not category:
            category = DEFAULT_CATEGORY
        else:
            category = VALID_KEY_RE.sub('', category).lower()

        if category not in CATEGORIES:
            self.logger.error("Database category '%s' does not exist.")
            return False

        now = datetime.now().strftime(SQLITE_DATE_FORMAT)

        # clean key
        key = VALID_KEY_RE.sub('', key).upper()
        if not key:
            return False

        # Get a cursor object
        cursor = self.socket.cursor()

        try:
            # First see if keystore already has a match
            if(bool(len(cursor.execute(
                "SELECT value FROM keystore WHERE " + \
                "container = ? AND category = ? AND key = ?",
                (self.container, category, key)).fetchall()))):
                if not cursor.execute(
                    "UPDATE keystore SET value = ?, last_update = ?" + \
                    " WHERE container = ? AND category = ? AND key = ?",
                    (value, now, self.container, category, key),):
                    return False
            else:
                if not cursor.execute(
                    "INSERT INTO keystore " + \
                    "(container, category, key, value, last_update) " + \
                    "VALUES (?, ?, ?, ?, ?)",
                    (self.container, category, key, value, now),):
                    return False
            # commit changes
            self.socket.commit()

        except sqlite3.OperationalError as e:
            # Database is corrupt or changed
            self.logger.debug(
                "Database.set() Operational Error: %s" % str(e))

            if not self._reset():
                self.logger.error(
                    "Detected damaged database; countermeasures failed.",
                )
                self.disabled = True

            else:
                self.logger.info(
                    "Detected damaged database; situation corrected.",
                )
            return False

        return True

    def get(self, key, default=None, category=None):
        """Get a value after specifying a key
        """

        if not self.socket:
            if not self.connect():
                return default

        if not category:
            category = DEFAULT_CATEGORY
        else:
            category = VALID_KEY_RE.sub('', category).lower()

        if category not in CATEGORIES:
            self.logger.error("Database category '%s' does not exist.")
            return default

        # clean key
        key = VALID_KEY_RE.sub('', key).upper()

        try:
            result = self.socket.execute(
                "SELECT value FROM keystore " + \
                "WHERE container = ? AND category = ? AND key = ?",
                (self.container, category, key),
            )
            if result:
                try:
                    return result.fetchall()[0][-1]
                except:
                    return default

        except sqlite3.OperationalError as e:
            # Database is corrupt or changed
            self.logger.debug(
                "Database.get() Operational Error: %s" % str(e))

            if not self._reset():
                self.logger.error(
                    "Detected damaged database; countermeasures failed.",
                )
                self.disabled = True

            else:
                self.logger.info(
                    "Detected damaged database; situation corrected.",
                )

        return default

    def items(self, category=None):
        """Return all variables as a list of tuples (k, p)
        """

        items = list()

        if not self.socket:
            if not self.connect():
                return items

        if not category:
            category = DEFAULT_CATEGORY
        else:
            category = VALID_KEY_RE.sub('', category).lower()

        if category not in CATEGORIES:
            self.logger.error("Database category '%s' does not exist.")
            return items

        # Get a cursor object
        cursor = self.socket.cursor()

        try:
            cursor.execute(
                "SELECT key, value FROM keystore " + \
                "WHERE container = ? AND category = ?",
                (self.container, category),
            )

        except sqlite3.OperationalError as e:
            # Database is corrupt or changed
            self.logger.debug(
                "Database.items() Operational Error: %s" % str(e))

            if not self._reset():
                self.logger.error(
                    "Detected damaged database; countermeasures failed.",
                )
                self.disabled = True

            else:
                self.logger.info(
                    "Detected damaged database; situation corrected.",
                )

            # early return of empty list
            return items

        while True:
            row = cursor.fetchone()
            if row is None:
                break

            items.append((row[0], row[1]))

        return items
